/*
 * $Id: //depot/prod/capstan/RCapstan_Hudson.2.0/src/java/com/netapp/dlm/repository/hibernate/LobCleanUpInterceptor.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.dlm.repository.hibernate;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.hibernate.Interceptor;
import net.sf.hibernate.type.Type;
import oracle.sql.BLOB;

import com.netapp.common.logging.NALogger;

/**
 * Frees temporary blobs after a flush to the database.
 * This frees up space in the temporary tablespace used by the blob.
 * 
 * @author philipc
 * @version $Revision: #1 $ $Date: 2018/05/31 $
 */
public class LobCleanUpInterceptor implements Interceptor {

    private static final NALogger logger = NALogger.getLogger(LobCleanUpInterceptor.class);

    private ThreadLocal<Collection<BLOB>> temporaryBlobs = new ThreadLocal<Collection<BLOB>>() {
        @Override
        protected Collection<BLOB> initialValue() {
            return new ArrayList<BLOB>();
        }
    };

    /**
     * @see net.sf.hibernate.Interceptor#onDelete(Object, Serializable id, Object[], String[], Type[])
     */
    public void onDelete(Object entity, Serializable id, Object[] state, String[] propertyNames, Type[] types) {
    }

    /**
     * @see net.sf.hibernate.Interceptor#onFlushDirty(Object, Serializable, Object[], Object[], String[], Type[])
     */
    public boolean onFlushDirty(Object entity, Serializable id, Object[] currentState, Object[] previousState,
        String[] propertyNames, Type[] types) {
        return false;
    }

    /**
     * @see net.sf.hibernate.Interceptor#onLoad(Object, Serializable, Object[], String[], Type[])
     */
    public boolean onLoad(Object entity, Serializable id, Object[] state, String[] propertyNames, Type[] types) {
        return false;
    }

    /**
     * @see net.sf.hibernate.Interceptor#onSave(Object, Serializable, Object[], String[], Type[])
     */
    public boolean onSave(Object entity, Serializable id, Object[] state, String[] propertyNames, Type[] types) {
        return false;
    }

    /**
     * @see net.sf.hibernate.Interceptor#postFlush(Iterator)
     */
    public void postFlush(Iterator entities) {
        for (Iterator<BLOB> iter = this.temporaryBlobs.get().iterator(); iter.hasNext();) {
            BLOB temporaryBlob = iter.next();
            try {
                if (temporaryBlob.isTemporary()) {
                    temporaryBlob.freeTemporary();
                }
            }
            catch (SQLException e) {
                logger.internal("Error freeing temporary space for BLOB", e);
            }
            iter.remove();
        }
    }

    /**
     * @see net.sf.hibernate.Interceptor#preFlush(Iterator)
     */
    public void preFlush(Iterator entities) {
    }

    /**
     * @see net.sf.hibernate.Interceptor#isUnsaved(java.lang.Object)
     */
    public Boolean isUnsaved(Object entity) {
        return Boolean.FALSE;
    }

    /**
     * @see net.sf.hibernate.Interceptor#instantiate(java.lang.Class, java.io.Serializable)
     */
    public Object instantiate(Class clazz, Serializable id) {
        return null;
    }

    /**
     * @see net.sf.hibernate.Interceptor#findDirty(Object, Serializable, Object[], Object[], String[], Type[])
     */
    public int[] findDirty(Object entity, Serializable id, Object[] currentState, Object[] previousState,
        String[] propertyNames, Type[] types) {
        return null;
    }

    public void registerTemporaryBlob(BLOB temporaryBlob) {
        this.temporaryBlobs.get().add(temporaryBlob);
    }

}
